/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse für einfache Programmieraufgaben mit mehreren Teilaufgaben
*/
function SqlCodingLmsConnector(
    node,       // DOM-Knoten, der für die Benutzeroberfläche genutzt werden soll
    levels      // Array describing levels
) {
    new ScoTools.InfoPageLink();
    this._ui = new SqlCodingTaskUI(node, levels);
    this._ui.onlevelunlock = (unlockedLevelCount => this._stateMgr && this._stateMgr.setValue('content.levelsCompleted', unlockedLevelCount));
    this._ui.onnewfine = (value => this._stateMgr && this._stateMgr.setValue('content.fine', value));
    this.connect(levels.length);
    this._ui.onlevelChange = (index => this._stateMgr && this._stateMgr.setValue('content.level', index));
    this._ui.oncodeChange = ((code) => this._stateMgr && this._stateMgr.setValue('content.userInput', code));
}

// Genutzte SqlCodingTaskUI-Instanz
SqlCodingLmsConnector.prototype._ui = null;

// Genutzte ScoStateMgr-Instanz
SqlCodingLmsConnector.prototype._stateMgr = null;


/**
* Versuchen, eine Verbindung zur Lernplattform aufzubauen und legt eine geeignet konfigurierte Instanz des ScoStateMgr an
* @return undefined
*/
SqlCodingLmsConnector.prototype.connect = function(
    countLevels     // Number of levels
) {
    const ScoStateMgr = ScoTools.ScoStateMgr;
    const fktValidateLevel = (v, prev) => Number.isFinite(+v) ? Math.max(0, Math.min(Math.round(+v), countLevels)) : prev;
    ScoStateMgr.registerInit(function() {
        this.registerKey('content.levelsCompleted', 0, fktValidateLevel, ScoStateMgr.TARGET_LOCATION, ScoStateMgr.PRIORITY_INDISPENSABLE, 0);
        this.registerKey('content.level', 0, fktValidateLevel, ScoStateMgr.TARGET_LOCATION, ScoStateMgr.PRIORITY_HOPEDFOR, 0);
        this.registerKey('content.fine', 0, v => Number.isFinite(+v) ? Math.max(0, Math.round(+v)) : Infinity, ScoStateMgr.TARGET_DATA, ScoStateMgr.PRIORITY_INDISPENSABLE, 0);
        this.registerKey('content.userInput', '', undefined, ScoStateMgr.TARGET_DATA, ScoStateMgr.PRIORITY_REQUIRED, 1);
//        this.registerKey('content.resultsTable', '', undefined, ScoStateMgr.TARGET_INPUT);
//        this.registerKey('content.selection', '0,0', undefined, ScoStateMgr.TARGET_INPUT);
//        this.registerKey('content.message', '', undefined, ScoStateMgr.TARGET_INPUT);
//        this.registerKey('content.lock', ?, undefined, ScoStateMgr.TARGET_INPUT);
        this.registerKey('content.storingMessage', 0, (v, prev) => ([0, 1, 2, 3].indexOf(v) == -1) ? prev : v, ScoStateMgr.TARGET_INPUT);
        this.registerKey('content.score', '', undefined, ScoStateMgr.TARGET_SCORE);
    });

    this._stateMgr = new ScoStateMgr();
    const ssm = this._stateMgr;
    const ui = this._ui;

    function updateStoringMsg() {
        if (!ssm.getValue('connection.lms'))
            return ssm.setValue('content.storingMessage', 1);
        if (ssm.getValue('pool4lms.unsatisfied') >= ScoStateMgr.PRIORITY_REQUIRED)
            return ssm.setValue('content.storingMessage', 2);
        if (['passed', 'completed'].indexOf(ssm.getValue('lms.lessonStatus')) != -1)
            return ssm.setValue('content.storingMessage', 3);
        ssm.setValue('content.storingMessage', 0);
    }
    ssm.addListener('content.storingMessage', index => ui.setStoringMsg([
        false,
        'Verbindungsaufnahme zur Lernplattform fehlgeschlagen - Fortschritte können nicht gespeichert werden',
        'Eingegebene SQL-Abfrage kann nicht gespeichert werden, weil sie zu lang ist',
        'Aufgabe erfolgreich abgeschlossen'
    ][index], (index == 3) ? ScoTools.MSGTYPE_SUCCESS : undefined, false));
    ssm.addListener('lms.lessonStatus', updateStoringMsg, false);
    ssm.addListener('pool4lms.unsatisfied', updateStoringMsg, false);
    ssm.addListener('connection.lms', updateStoringMsg, true);

    function recalculateScore() {
        const levelsCompleted = ssm.getValue('content.levelsCompleted');
        const done = (levelsCompleted >= countLevels);
        ssm.setValue('content.score', levelsCompleted + (done ? Math.max(0, 100 - countLevels - ssm.getValue('content.fine')) : 0));
        ssm.setValue('lms.lessonStatus', done ? 'completed' : 'incomplete');
    };
    ssm.addListener('content.fine', value => ui.fineCount = value);
    ssm.addListener('content.levelsCompleted', value => { ui.unlockedLevelCount = value; recalculateScore(); });
    ssm.addListener('content.level', index => ui.levelIndex = index);
    ssm.addListener('content.userInput', s => ui.userInput = '' + s);
}
